import os
import unittest
import deprecation

from io import BytesIO
from .. import DATA_D

import eyed3


def testvalidHeader():
    from eyed3.mp3.headers import isValidHeader

    # False sync, the layer is invalid
    assert not isValidHeader(0xffe00000)
    # False sync, bitrate is invalid
    assert not isValidHeader(0xffe20000)
    assert not isValidHeader(0xffe20001)
    assert not isValidHeader(0xffe2000f)
    # False sync, sample rate is invalid
    assert not isValidHeader(0xffe21c34)
    assert not isValidHeader(0xffe21c54)
    # False sync, version is invalid
    assert not isValidHeader(0xffea0000)
    assert not isValidHeader(0xffea0001)
    assert not isValidHeader(0xffeb0001)
    assert not isValidHeader(0xffec0001)


    assert not isValidHeader(0)
    assert not isValidHeader(0xffffffff)
    assert not isValidHeader(0xffe0ffff)
    assert not isValidHeader(0xffe00000)
    assert not isValidHeader(0xfffb0000)

    assert isValidHeader(0xfffb9064)
    assert isValidHeader(0xfffb9074)
    assert isValidHeader(0xfffb900c)
    assert isValidHeader(0xfffb1900)
    assert isValidHeader(0xfffbd204)
    assert isValidHeader(0xfffba040)
    assert isValidHeader(0xfffba004)
    assert isValidHeader(0xfffb83eb)
    assert isValidHeader(0xfffb7050)
    assert isValidHeader(0xfffb32c0)


def testFindHeader():
    from eyed3.mp3.headers import findHeader

    # No header
    buffer = BytesIO(b'\x00' * 1024)
    (offset, header_int, header_bytes) = findHeader(buffer, 0)
    assert header_int is None

    # Valid header
    buffer = BytesIO(b'\x11\x12\x23' * 1024 + b"\xff\xfb\x90\x64" +
                     b"\x00" * 1024)
    (offset, header_int, header_bytes) = findHeader(buffer, 0)
    assert header_int == 0xfffb9064

    # Same thing with a false sync in the mix
    buffer = BytesIO(b'\x11\x12\x23' * 1024 +
                     b"\x11" * 100 +
                     b"\xff\xea\x00\x00" + # false sync
                     b"\x22" * 100 +
                     b"\xff\xe2\x1c\x34" + # false sync
                     b"\xee" * 100 +
                     b"\xff\xfb\x90\x64" +
                     b"\x00" * 1024)
    (offset, header_int, header_bytes) = findHeader(buffer, 0)
    assert header_int == 0xfffb9064


@unittest.skipIf(not os.path.exists(DATA_D), "test requires data files")
def testBasicVbrMp3():
    audio_file = eyed3.load(os.path.join(DATA_D, "notag-vbr.mp3"))
    assert isinstance(audio_file, eyed3.mp3.Mp3AudioFile)

    assert audio_file.info is not None
    assert round(audio_file.info.time_secs) == 262
    assert audio_file.info.size_bytes == 6272220
    # Variable bit rate, ~191
    assert audio_file.info.bit_rate[0] == True
    assert audio_file.info.bit_rate[1] == 191
    assert audio_file.info.bit_rate_str == "~191 kb/s"

    assert audio_file.info.mode == "Joint stereo"
    assert audio_file.info.sample_freq == 44100

    assert audio_file.info.mp3_header is not None
    assert audio_file.info.mp3_header.version == 1.0
    assert audio_file.info.mp3_header.layer == 3

    assert audio_file.info.xing_header is not None
    assert audio_file.info.lame_tag is not None
    assert audio_file.info.vbri_header is None
    assert audio_file.tag is None


@deprecation.fail_if_not_removed
def test_compute_time_from_frame_deprecation():
    from eyed3.mp3.headers import compute_time_per_frame

    compute_time_per_frame(None)




from io import StringIO
import eyed3
import os
import sys
import logging
import unittest

DATA_D = os.path.join(os.path.dirname(__file__), "data")

eyed3.log.setLevel(logging.ERROR)


class RedirectStdStreams(object):
    """This class is used to capture sys.stdout and sys.stderr for tests that
    invoke command line scripts and wish to inspect the output."""

    def __init__(self, stdout=None, stderr=None, seek_on_exit=0):
        self.stdout = stdout or StringIO()
        self.stderr = stderr or StringIO()
        self._seek_offset = seek_on_exit

    def __enter__(self):
        self._orig_stdout, self._orig_stderr = sys.stdout, sys.stderr
        sys.stdout, sys.stderr = self.stdout, self.stderr
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        try:
            for s in [self.stdout, self.stderr]:
                s.flush()
                if not s.isatty():
                    s.seek(self._seek_offset)
        finally:
            sys.stdout, sys.stderr = self._orig_stdout, self._orig_stderr


class ExternalDataTestCase(unittest.TestCase):
    """Test case for external data files."""
    def setUp(self):
        pass



"""Tests for eyed3.utils.console module"""
import unittest
from unittest import mock
from eyed3.utils.console import AnsiCodes, Fore


@mock.patch('sys.stdout.isatty', new=lambda: True)
class AnsiCodesTC(unittest.TestCase):
    def setUp(self):
        AnsiCodes._USE_ANSI = False

    def test_init_color_enabled(self):
        AnsiCodes.init(True)
        self._assert_color_enabled()

    def test_init_color_disabled(self):
        AnsiCodes.init(False)
        self._assert_color_disabled()

    @mock.patch('sys.stdout.isatty', new=lambda: False)
    def test_init_color_enabled_not_tty(self):
        AnsiCodes.init(False)
        self._assert_color_disabled()

    def _assert_color_enabled(self):
        self.assertTrue(AnsiCodes._USE_ANSI)
        self.assertEqual(Fore.GREEN, '\x1b[32m')

    def _assert_color_disabled(self):
        self.assertFalse(AnsiCodes._USE_ANSI)
        self.assertEqual(Fore.GREEN, '')



from eyed3.plugins import *


def test_load():
    plugins = load()
    assert "classic" in  list(plugins.keys())
    assert "genres" in list(plugins.keys())

    assert load("classic") == plugins["classic"]
    assert load("genres") == plugins["genres"]

    assert (load("classic", reload=True).__class__.__name__ ==
            plugins["classic"].__class__.__name__)
    assert (load("genres", reload=True).__class__.__name__ ==
            plugins["genres"].__class__.__name__)

    assert load("DNE") is None

def test_Plugin():
    import argparse
    class MyPlugin(Plugin):
        pass

    p = MyPlugin(argparse.ArgumentParser())
    assert p.arg_group is not None

    # In reality, this is parsed args
    p.start("dummy_args", "dummy_config")
    assert p.args == "dummy_args"
    assert p.config == "dummy_config"

    assert p.handleFile("f.txt") is None
    assert p.handleDone() is None




import os
import pytest
import deprecation

from eyed3 import utils, mimetype
from . import DATA_D


mime_test_params = [("id3", ["application/x-id3"]),
                    ("tag", ["application/x-id3"]),
                    ("mka", ["video/x-matroska", "application/octet-stream"]),
                    ("mp3", ["audio/mpeg"]),
                    ("ogg", ["audio/ogg", "application/ogg"]),
                    ("wav", ["audio/x-wav"]),
                    ("wma", ["audio/x-ms-wma", "video/x-ms-wma", "video/x-ms-asf",
                             "video/x-ms-wmv"]),
                   ]


@pytest.mark.skipif(not os.path.exists(DATA_D), reason="test requires data files")
@deprecation.fail_if_not_removed
def testSampleMimeTypesUtils():
    for ext, valid_types in mime_test_params:
        guessed = utils.guessMimetype(os.path.join(DATA_D, f"sample.%s" % ext))
        assert guessed in valid_types


@pytest.mark.skipif(not os.path.exists(DATA_D), reason="test requires data files")
@pytest.mark.parametrize(("ext", "valid_types"), mime_test_params)
def testSampleMimeTypes(ext, valid_types):
    guessed = mimetype.guessMimetype(os.path.join(DATA_D, "sample.%s" % ext))
    assert guessed in valid_types




from unittest.mock import MagicMock, call

import eyed3.utils.console
from eyed3.utils import walk
from eyed3.utils.console import (
    printMsg, printWarning, printHeader, Fore, WARNING_COLOR, HEADER_COLOR
)
from . import RedirectStdStreams


def test_printWarning():
    eyed3.utils.console.USE_ANSI = False
    with RedirectStdStreams() as out:
        printWarning("Built To Spill")
    assert (out.stdout.read() == "Built To Spill\n")

    eyed3.utils.console.USE_ANSI = True
    with RedirectStdStreams() as out:
        printWarning("Built To Spill")
    assert (out.stdout.read() == "%sBuilt To Spill%s\n" % (WARNING_COLOR(),
                                                              Fore.RESET))


def test_printMsg():
    eyed3.utils.console.USE_ANSI = False
    with RedirectStdStreams() as out:
        printMsg("EYEHATEGOD")
    assert (out.stdout.read() == "EYEHATEGOD\n")

    eyed3.utils.console.USE_ANSI = True
    with RedirectStdStreams() as out:
        printMsg("EYEHATEGOD")
    assert (out.stdout.read() == "EYEHATEGOD\n")


def test_printHeader():
    eyed3.utils.console.USE_ANSI = False
    with RedirectStdStreams() as out:
        printHeader("Furthur")
    assert (out.stdout.read() == "Furthur\n")

    eyed3.utils.console.USE_ANSI = True
    with RedirectStdStreams() as out:
        printHeader("Furthur")
    assert (out.stdout.read() == "%sFurthur%s\n" % (HEADER_COLOR(),
                                                       Fore.RESET))


def test_walk_recursive(tmpdir):
    root_d = tmpdir.mkdir("Root")
    d1 = root_d.mkdir("d1")
    f1 = d1 / "file1"
    f1.write_text("file1", "utf8")

    _ = root_d.mkdir("d2")
    d3 = root_d.mkdir("d3")

    handler = MagicMock()
    walk(handler, str(root_d), recursive=True)
    handler.handleFile.assert_called_with(str(f1))
    handler.handleDirectory.assert_called_with(str(d1), [f1.basename])

    # Only dirs with files are handled, so...
    f2 = d3 / "Neurosis"
    f2.write_text("Through Silver and Blood", "utf8")
    f3 = d3 / "High on Fire"
    f3.write_text("Surrounded By Thieves", "utf8")

    d4 = d3.mkdir("d4")
    f4 = d4 / "Cross Rot"
    f4.write_text("VII", "utf8")

    handler = MagicMock()
    walk(handler, str(root_d), recursive=True)
    handler.handleFile.assert_has_calls([call(str(f1)),
                                         call(str(f3)),
                                         call(str(f2)),
                                         call(str(f4)),
                                        ], any_order=True)
    handler.handleDirectory.assert_has_calls(
        [call(str(d1), [f1.basename]),
         call(str(d3), [f3.basename, f2.basename]),
         call(str(d4), [f4.basename]),
        ], any_order=True)


def test_walk(tmpdir):
    root_d = tmpdir.mkdir("Root")
    d1 = root_d.mkdir("d1")
    f1 = d1 / "file1"
    f1.write_text("file1", "utf8")

    _ = root_d.mkdir("d2")
    d3 = root_d.mkdir("d3")

    f2 = d3 / "Neurosis"
    f2.write_text("Through Silver and Blood", "utf8")
    f3 = d3 / "High on Fire"
    f3.write_text("Surrounded By Thieves", "utf8")

    d4 = d3.mkdir("d4")
    f4 = d4 / "Cross Rot"
    f4.write_text("VII", "utf8")

    handler = MagicMock()
    walk(handler, str(root_d))
    handler.handleFile.assert_not_called()
    handler.handleDirectory.assert_not_called()

    handler = MagicMock()
    walk(handler, str(root_d / "d1"), recursive=True)
    handler.handleFile.assert_called_with(str(f1))
    handler.handleDirectory.assert_called_with(str(d1), [f1.basename])

    handler = MagicMock()
    walk(handler, str(root_d / "d3"))
    handler.handleFile.assert_has_calls([call(str(f3)), call(str(f2))], any_order=True)
    handler.handleDirectory.assert_has_calls([call(str(d3), [f3.basename, f2.basename])],
                                             any_order=True)



#!/usr/bin/env python
from eyed3.id3 import Tag
from eyed3.id3 import ID3_V1_0, ID3_V1_1, ID3_V2_3, ID3_V2_4

import logging
from eyed3 import log
log.setLevel(logging.DEBUG)

t = Tag()
t.artist = "M.O.P."
t.title = "How About Some Hardcore"
t.album = "To The Death"
t.genre = "Hip-Hop"
t.track_num = (3, 5)
t.disc_num = (1, 1)

t.original_release_date = "1994-04-07"
t.release_date = "1994-04-07"
t.encoding_date = "2002-03"
t.recording_date = 1996
t.tagging_date = "2012-2-5"

t.comments.set("Gritty, yo!")
t.comments.set("Brownsville, Brooklyn", "Origin")

t.user_text_frames.set("****", "Rating")
t.artist_url = b"http://allmusic.com/artist/mop-p194909"
t.user_url_frames.set(b"http://eyed3.nicfit.net/")

t.bpm = 187
t.play_count = 125
t.unique_file_ids.set(b"43e888e067ea107f964916af6259cbe7", "md5sum")
t.cd_id = b"\x3c\x33\x4d\x41\x43\x59\x3c\x33"
t.privates.set(b"Secrets", b"Billy Danzenie")
t.terms_of_use = "Blunted"
t.lyrics.set("""
[ Billy Danzenie ]
How about some hardcore?
(Yeah, we like it raw!)  (4x)
How about some hardcore?

[ VERSE 1: Billy Danzenie ]
(Yeah, we like it raw in the streets)
For the fellas on the corner posted up 20 deep
With your ifth on your hip, ready to flip
Whenever you empty your clip, dip, trip your sidekick
You got skill, you best manage to chill
And do yourself a favor, don`t come nowhere near the Hill
With that bullshit, word, money grip, it`ll cost ya
Make you reminisce of Frank Nitty `The Enforcer`
I move with M.O.P.`s Last Generation
Straight up and down, act like you want a confrontation
I packs my gat, I gotta stay strapped
I bust mines, don`t try to sneak up on me from behind
Don`t sleep, I get deep when I creep
I see right now I got to show you it ain`t nothin sweet
Go get your muthaf**kin hammer
And act like you want drama
I send a message to your mama
`Hello, do you know your one son left?
I had license to kill and he had been marked for death
He`s up the Hill in the back of the building with two in the dome
I left him stiffer than a tombstone`

[ Li`l Fame ]
How about some hardcore?
(Yeah, we like it raw!)  (4x)
How about some hardcore?

[ VERSE 2: Billy Danzenie ]
(Yeah, we like it rugged in the ghetto)
I used to pack sling shots, but now I`m packin heavy metal
A rugged underground freestyler
Is Li`l Fame, muthaf**ka, slap, Li`l Mallet
When I let off, it`s a burning desire
Niggas increase the peace cause when I release it be rapid fire
For the cause I drop niggas like drawers
Niggas`ll hit the floors from the muthaf**kin .44`s
I`m talkin titles when it`s showtime
f**k around, I have niggas call the injury help line
I bust words in my verse that`ll serve
Even on my first nerve I put herbs to curbs
I ain`t about givin niggas a chance
And I still raise sh*t to make my brother wanna get up and dance
Front, I make it a thrill to kill
Bringin the ruckus, it`s the neighborhood hoods for the Hill that`s real
Me and mics, that`s unlike niggas and dykes
So who wanna skate, cause I`m puttin niggas on ice
Whatever I drop must be rough, rugged and hard more
(Yeah!)

[ Billy Danzenie ]
How about some hardcore?
(Yeah, we like it raw!)  (4x)

[ VERSE 3: Billy Danzenie ]
Yo, here I am (So what up?) Get it on, cocksucker
That nigga Bill seem to be a ill black brother
I gets dough from the way I flow
And before I go
You muthaf**kas gonna know
That I ain`t nothin to f**k with - duck quick
I squeeze when I`m stressed
Them teflons`ll tear through your vest
I love a bloodbath (niggas know the half)
You can feel the wrath (Saratoga/St. Marks Ave.)
B-i-l-l-y D-a-n-z-e
n-i-e, me, Billy Danzenie
(Knock, knock) Who`s there? (Li`l Fame)
Li`l Fame who? (Li`l Fame, your nigga)
Boom! Ease up off the trigger
It`s aight, me and shorty go to gunfights
Together we bring the ruckus, right?
We trump tight, aight?
I earned mine, so I`m entitled to a title
(7 f**kin 30) that means I`m homicidal

[ Li`l Fame ]
How about some hardcore?
(Yeah, we like it raw!)  (4x)

[ VERSE 4: Li`l Fame ]
Yo, I scream on niggas like a rollercoaster
To them wack muthaf**kas, go hang it up like a poster
Niggas get excited, but don`t excite me
Don`t invite me, I`m splittin niggas` heads where the white be
Try to trash this, this little bastard`ll blast it
Only puttin niggas in comas and caskets
I ain`t a phoney, I put the `mack` in a -roni
I leave you lonely (Yeah, yeah, get on his ass, homie)
Up in your anus, I pack steel that`s stainless
We came to claim this, and Li`l Fame`ll make you famous
I mack hoes, rock shows and stack dough
Cause I`m in effect, knockin muthaf**kas like five-o
I`m catchin other niggas peepin, shit, I ain`t sleepin
I roll deep like a muthaf**kin Puerto-Rican
So when I write my competition looks sadly
For broke-ass niggas I make it happen like Mariah Carey
I got sh*t for niggas that roll bold
Li`l Fame is like a orthopedic shoe, I got mad soul
I`ma kill em before I duck em
Because yo, mother made em, mother had em and muthaf**k em

[ Li`l Fame ]
Knowmsayin?
Li`l Fame up in this muthaf**ka
Givin shoutouts to my man D/R Period
[Name]
Lazy Laz
My man Broke As* Moe
The whole Saratoga Ave.
Youknowmsayin?
Representin for Brooklyn
Most of all my cousin Prince Leroy, Big Mal, rest in peace
[ Billy Danzenie ]
Danzenie up in this muthaf**ka
I`d like to say what`s up to the whole M.O.P.
Brooklyn, period
Them niggas that just don`t give a f**k
[ O.G. Bu-Bang
Bet yo ass, nigga
Hey yo, this muthaf**kin Babyface [Name]
Aka O.G. Bu-Bang
Yo, I wanna say what`s up to the whole muthaf**kin M.O.P. boyyeee
""")

t.save("example-v2_4.id3", version=ID3_V2_4)
t.save("example-v2_3.id3", version=ID3_V2_3)

# Loss of the release date month and day.
# Loss of the comment with description.
t.save("example-v1_1.id3", version=ID3_V1_1)

# Loses what v1.1 loses, and the track #
t.save("example-v1_0.id3", version=ID3_V1_0)

'''
from eyed3.id3.tag import TagTemplate
template = "$artist/"\
           "$best_release_date:year - $album/"\
           "$artist - $track:num - $title.$file:ext"
print TagTemplate(template).substitute(t, zeropad=True)
'''



import eyed3
import unittest
import pytest
from pathlib import Path
from eyed3.id3 import *
from .. import DATA_D

ID3_VERSIONS = [(ID3_V1, (1, None, None), "v1.x"),
                (ID3_V1_0, (1, 0, 0), "v1.0"),
                (ID3_V1_1, (1, 1, 0), "v1.1"),
                (ID3_V2, (2, None, None), "v2.x"),
                (ID3_V2_2, (2, 2, 0), "v2.2"),
                (ID3_V2_3, (2, 3, 0), "v2.3"),
                (ID3_V2_4, (2, 4, 0), "v2.4"),
                (ID3_DEFAULT_VERSION, (2, 4, 0), "v2.4"),
                (ID3_ANY_VERSION, (1|2, None, None), "v1.x/v2.x"),
                ]

with pytest.raises(TypeError):
    versionToString(666)

with pytest.raises(ValueError):
    versionToString((3, 1, 0))


def testEmptyGenre():
    g = Genre()
    assert g.id is None
    assert g.name is None


def testValidGenres():
    # Create with id
    for i in range(genres.GENRE_MAX):
        g = Genre()
        g.id = i
        assert g.id == i
        assert g.name == genres[i]

        g = Genre(id=i)
        assert g.id == i
        assert g.name == genres[i]

    # Create with name
    for name in [n for n in genres if n is not None and type(n) is not int]:
        g = Genre()
        g.name = name
        assert g.id == genres[name]
        assert g.name == genres[g.id]
        assert g.name.lower() == name

        g = Genre(name=name)
        assert g.id == genres[name]
        assert g.name.lower() == name


def test255Padding():
    for i in range(GenreMap.GENRE_MAX + 1, 256):
        assert genres[i] is None
    with pytest.raises(KeyError):
        genres.__getitem__(256)


def testCustomGenres():
    # Genres can be created for any name, their ID is None
    g = Genre(name="Grindcore")
    assert g.name == "Grindcore"
    assert g.id is None

    # But when constructing with IDs they must map.
    with pytest.raises(ValueError):
        Genre.__call__(id=1024)


def testRemappedNames():
    g = Genre(id=3, name="dance stuff")
    assert g.id == 3
    assert g.name == "Dance"

    g = Genre(id=666, name="Funky")
    assert g.id is None
    assert g.name == "Funky"


def testGenreEq():
    for s in ["Hardcore", "(129)Hardcore",
              "(129)", "(0129)",
              "129", "0129"]:
        assert Genre.parse(s) == Genre.parse(s)
        assert Genre.parse(s) != Genre.parse("Blues")


def testParseGenre():
    test_list = ["Hardcore", "(129)Hardcore",
                 "(129)", "(0129)",
                 "129", "0129"]

    # This is typically what will happen when parsing tags, a blob of text
    # is parsed into Genre
    for s in test_list:
        g = Genre.parse(s)
        assert g.name == "Hardcore"
        assert g.id == 129

    g = Genre.parse("")
    assert g is None

    g = Genre.parse("1")
    assert g.id == 1
    assert g.name == "Classic Rock"

    g = Genre.parse("1", id3_std=False)
    assert g.id is None
    assert g.name == "1"


def testToSting():
    assert str(Genre("Hardcore")) == "(129)Hardcore"
    assert str(Genre("Grindcore")) == "Grindcore"


def testId3Versions():
    for v in [ID3_V1, ID3_V1_0, ID3_V1_1]:
        assert (v[0] == 1)

    assert (ID3_V1_0[1] == 0)
    assert (ID3_V1_0[2] == 0)
    assert (ID3_V1_1[1] == 1)
    assert (ID3_V1_1[2] == 0)

    for v in [ID3_V2, ID3_V2_2, ID3_V2_3, ID3_V2_4]:
        assert (v[0] == 2)

    assert (ID3_V2_2[1] == 2)
    assert (ID3_V2_3[1] == 3)
    assert (ID3_V2_4[1] == 4)

    assert (ID3_ANY_VERSION == (ID3_V1[0] | ID3_V2[0], None, None))
    assert (ID3_DEFAULT_VERSION == ID3_V2_4)


def test_versionToString():
    for const, tple, string in ID3_VERSIONS:
        assert versionToString(const) == string


def test_isValidVersion():
    for v, _, _ in ID3_VERSIONS:
        assert isValidVersion(v)

    for _, v, _ in ID3_VERSIONS:
        if None in v:
            assert not isValidVersion(v, True)
        else:
            assert isValidVersion(v, True)

    assert not isValidVersion((3, 1, 1))


def testNormalizeVersion():
    assert normalizeVersion(ID3_V1) == ID3_V1_1
    assert normalizeVersion(ID3_V2) == ID3_V2_4
    assert normalizeVersion(ID3_DEFAULT_VERSION) == ID3_V2_4
    assert normalizeVersion(ID3_ANY_VERSION) == ID3_DEFAULT_VERSION
    # Correcting the bogus
    assert normalizeVersion((2, 2, 1)) == ID3_V2_2


# ID3 v2.2
@unittest.skipIf(not Path(DATA_D).exists(), "test requires data files")
def test_id3v22():
    data_file = Path(DATA_D) / "sample-ID3v2.2.0.tag"
    audio_file = eyed3.load(data_file)
    assert audio_file.tag.version == (2, 2, 0)
    assert audio_file.tag.title == "11.Portfolio Diaz.mp3"
    assert audio_file.tag.album == "Acrobatic Tenement"
    assert audio_file.tag.artist == "At the Drive-In"



import unittest
from pathlib import Path

from eyed3 import main
from . import DATA_D, RedirectStdStreams


@unittest.skipIf(not Path(DATA_D).exists(), "test requires data files")
def testLameInfoPlugin():
    test_file = Path(DATA_D) / "mp3_samples/mpeg2.5 12.000kHz __vbr__ simple.mp3"
    with RedirectStdStreams() as plugin_out:
        args, _, config = main.parseCommandLine(["-P", "lameinfo", str(test_file)])
        retval = main.main(args, config)
        assert retval == 0

    stdout = plugin_out.stdout.read()
    assert stdout[stdout.index("Encoder Version"):].strip() == \
"""
Encoder Version     : LAME3.99r
LAME Tag Revision   : 0
VBR Method          : Variable Bitrate method2 (mtrh)
Lowpass Filter      : 6000
Radio Replay Gain   : 12.2 dB (Set automatically)
Encoding Flags      : --nspsytune --nssafejoint
ATH Type            : 5
Bitrate (Minimum)   : 8
Encoder Delay       : 576 samples
Encoder Padding     : 960 samples
Noise Shaping       : 1
Stereo Mode         : Stereo
Unwise Settings     : False
Sample Frequency    : 44.1 kHz
MP3 Gain            : 0 (+0.0 dB)
Preset              : V0
Surround Info       : None
Music Length        : 67.88 KB
Music CRC-16        : 8707
LAME Tag CRC-16     : 0000
""".strip()


@unittest.skipIf(not Path(DATA_D).exists(), "test requires data files")
def testLameInfoPlugin_None():
    test_file = Path(DATA_D) / "test.mp3"
    with RedirectStdStreams() as plugin_out:
        args, _, config = main.parseCommandLine(["-P", "lameinfo", str(test_file)])
        retval = main.main(args, config)
        assert retval == 0

    stdout = plugin_out.stdout.read()
    assert stdout[stdout.index("--\n") + 3:].strip() == "No LAME Tag"




import unittest
import pytest
from eyed3.id3 import TagFile
from eyed3.plugins.display import *


class TestDisplayPlugin(unittest.TestCase):

    def __init__(self, name):
        super(TestDisplayPlugin, self).__init__(name)

    def testSimpleTags(self):
        self.file.tag.artist = "The Artist"
        self.file.tag.title = "Some Song"
        self.file.tag.composer = "Some Composer"
        self.__checkOutput("%a% - %t% - %C%", "The Artist - Some Song - Some Composer")

    def testComposer(self):
        self.file.tag.composer = "Bad Brains"
        self.__checkOutput("%C% - %composer%", "Bad Brains - Bad Brains")

    def testCommentsTag(self):
        self.file.tag.comments.set("TEXT", description="", lang=b"DE")
        self.file.tag.comments.set("#d-tag", description="#l-tag", lang=b"#t-tag")
        # Langs are chopped to 3 bytes (are are codes), so #t- is expected.
        self.__checkOutput("%comments,output=#d #l #t,separation=|%", " DE TEXT|#l-tag #t- #d-tag")

    def testRepeatFunction(self):
        self.__checkOutput("$repeat(*,3)", "***")
        self.__checkException("$repeat(*,three)", DisplayException)

    def testNotEmptyFunction(self):
        self.__checkOutput("$not-empty(foo,hello #t,nothing)", "hello foo")
        self.__checkOutput("$not-empty(,hello #t,nothing)", "nothing")

    def testNumberFormatFunction(self):
        self.__checkOutput("$num(123,5)", "00123")
        self.__checkOutput("$num(123,3)", "123")
        self.__checkOutput("$num(123,0)", "123")
        self.__checkException("$num(nan,1)", DisplayException)
        self.__checkException("$num(1,foo)", DisplayException)
        self.__checkException("$num(1,)", DisplayException)

    def __checkOutput(self, pattern, expected):
        output = Pattern(pattern).output_for(self.file)
        assert output == expected

    def __checkException(self, pattern, exception_type):
        with pytest.raises(exception_type):
            Pattern(pattern).output_for(self.file)

    def setUp(self):
        import tempfile
        with tempfile.NamedTemporaryFile() as temp:
            temp.flush()
            self.file = TagFile(temp.name)
        self.file.initTag()

    def tearDown(self):
        pass


class TestDisplayParser(unittest.TestCase):

    def __init__(self, name):
        super(TestDisplayParser, self).__init__(name)

    def testTextPattern(self):
        pattern = Pattern("hello")
        assert isinstance(pattern.sub_patterns[0], TextPattern)
        assert len(pattern.sub_patterns) == 1

    def testTagPattern(self):
        pattern = Pattern("%comments,desc,lang,separation=|%")
        assert len(pattern.sub_patterns) == 1
        assert isinstance(pattern.sub_patterns[0], TagPattern)
        comments_tag = pattern.sub_patterns[0]
        assert (len(comments_tag.parameters) == 4)
        assert comments_tag._parameter_value("description", None) == "desc"
        assert comments_tag._parameter_value("language", None) == "lang"
        assert (comments_tag._parameter_value("output", None) ==
                AllCommentsTagPattern.PARAMETERS[2].default)
        assert comments_tag._parameter_value("separation", None) == "|"

    def testComplexPattern(self):
        pattern = Pattern("Output: $format(Artist: $not-empty(%artist%,#t,none),bold=y)")
        assert len(pattern.sub_patterns) == 2
        assert isinstance(pattern.sub_patterns[0], TextPattern)
        assert isinstance(pattern.sub_patterns[1], FunctionFormatPattern)
        text_patten = pattern.sub_patterns[1].parameters['text'].value
        assert len(text_patten.sub_patterns) == 2
        assert isinstance(text_patten.sub_patterns[0], TextPattern)
        assert isinstance(text_patten.sub_patterns[1], FunctionNotEmptyPattern)

    def testCompileException(self):
        with pytest.raises(PatternCompileException):
            Pattern("$bad-pattern").output_for(None)
        with pytest.raises(PatternCompileException):
            Pattern("$unknown-function()").output_for(None)

    def setUp(self):
        pass

    def tearDown(self):
        pass



import unittest
import pytest
from eyed3.id3.headers import *
from eyed3.id3 import ID3_DEFAULT_VERSION, TagException

from io import BytesIO


class TestTagHeader(unittest.TestCase):
    def testCtor(self):
        h = TagHeader()
        assert (h.version == ID3_DEFAULT_VERSION)
        assert not(h.unsync)
        assert not(h.extended)
        assert not(h.experimental)
        assert not(h.footer)
        assert h.tag_size == 0

    def testTagVersion(self):
        for maj, min, rev in [(1, 0, 0), (1, 1, 0), (2, 2, 0), (2, 3, 0),
                              (2, 4, 0)]:
            h = TagHeader((maj, min, rev))

            assert (h.major_version == maj)
            assert (h.minor_version == min)
            assert (h.rev_version == rev)

        for maj, min, rev in [(1, 0, None), (1, None, 0), (2, 5, 0), (3, 4, 0)]:
            try:
                h = TagHeader((maj, min, rev))
            except ValueError:
                pass
            else:
                assert not("Invalid version, expected ValueError")

    def testParse(self):
        # Incomplete headers
        for data in [b"", b"ID3", b"ID3\x04\x00",
                     b"ID3\x02\x00\x00",
                     b"ID3\x03\x00\x00",
                     b"ID3\x04\x00\x00",
                    ]:
            header = TagHeader()
            found = header.parse(BytesIO(data))
            assert not(found)

        # Invalid versions
        for data in [b"ID3\x01\x00\x00",
                     b"ID3\x05\x00\x00",
                     b"ID3\x06\x00\x00",
                    ]:
            header = TagHeader()
            try:
                found = header.parse(BytesIO(data))
            except TagException:
                pass
            else:
                assert not("Expected TagException invalid version")


        # Complete headers
        for data in [b"ID3\x02\x00\x00",
                     b"ID3\x03\x00\x00",
                     b"ID3\x04\x00\x00",
                    ]:
            for sz in [0, 10, 100, 1000, 2500, 5000, 7500, 10000]:
                sz_bytes = bin2bytes(bin2synchsafe(dec2bin(sz, 32)))
                header = TagHeader()
                found = header.parse(BytesIO(data + sz_bytes))
                assert (found)
                assert header.tag_size == sz

    def testRenderWithUnsyncTrue(self):
        h = TagHeader()
        h.unsync = True
        with pytest.raises(NotImplementedError):
            h.render(100)

    def testRender(self):
        h = TagHeader()
        h.unsync = False
        header = h.render(100)

        h2 = TagHeader()
        found = h2.parse(BytesIO(header))
        assert not(h2.unsync)
        assert (found)
        assert header == h2.render(100)

        h = TagHeader()
        h.footer = True
        h.extended = True
        header = h.render(666)

        h2 = TagHeader()
        found = h2.parse(BytesIO(header))
        assert (found)
        assert not(h2.unsync)
        assert not(h2.experimental)
        assert h2.footer
        assert h2.extended
        assert (h2.tag_size == 666)
        assert (header == h2.render(666))

class TestExtendedHeader(unittest.TestCase):
    def testCtor(self):
        h = ExtendedTagHeader()
        assert (h.size == 0)
        assert (h._flags == 0)
        assert (h.crc is None)
        assert (h._restrictions == 0)

        assert not(h.update_bit)
        assert not(h.crc_bit)
        assert not(h.restrictions_bit)

    def testUpdateBit(self):
        h = ExtendedTagHeader()

        h.update_bit = 1
        assert (h.update_bit)
        h.update_bit = 0
        assert not(h.update_bit)
        h.update_bit = 1
        assert (h.update_bit)
        h.update_bit = False
        assert not(h.update_bit)
        h.update_bit = True
        assert (h.update_bit)

    def testCrcBit(self):
        h = ExtendedTagHeader()
        h.update_bit = True

        h.crc_bit = 1
        assert (h.update_bit)
        assert (h.crc_bit)
        h.crc_bit = 0
        assert (h.update_bit)
        assert not(h.crc_bit)
        h.crc_bit = 1
        assert (h.update_bit)
        assert (h.crc_bit)
        h.crc_bit = False
        assert (h.update_bit)
        assert not(h.crc_bit)
        h.crc_bit = True
        assert (h.update_bit)
        assert (h.crc_bit)

    def testRestrictionsBit(self):
        h = ExtendedTagHeader()
        h.update_bit = True
        h.crc_bit = True

        h.restrictions_bit = 1
        assert (h.update_bit)
        assert (h.crc_bit)
        assert (h.restrictions_bit)
        h.restrictions_bit = 0
        assert (h.update_bit)
        assert (h.crc_bit)
        assert not(h.restrictions_bit)
        h.restrictions_bit = 1
        assert (h.update_bit)
        assert (h.crc_bit)
        assert (h.restrictions_bit)
        h.restrictions_bit = False
        assert (h.update_bit)
        assert (h.crc_bit)
        assert not(h.restrictions_bit)
        h.restrictions_bit = True
        assert (h.update_bit)
        assert (h.crc_bit)
        assert (h.restrictions_bit)

        h = ExtendedTagHeader()
        h.restrictions_bit = True
        assert (h.tag_size_restriction ==
                ExtendedTagHeader.RESTRICT_TAG_SZ_LARGE)
        assert (h.text_enc_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_ENC_NONE)
        assert (h.text_length_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_LEN_NONE)
        assert (h.image_enc_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_ENC_NONE)
        assert (h.image_size_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_SZ_NONE)

        h.tag_size_restriction = ExtendedTagHeader.RESTRICT_TAG_SZ_TINY
        assert (h.tag_size_restriction ==
                ExtendedTagHeader.RESTRICT_TAG_SZ_TINY)
        assert (h.text_enc_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_ENC_NONE)
        assert (h.text_length_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_LEN_NONE)
        assert (h.image_enc_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_ENC_NONE)
        assert (h.image_size_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_SZ_NONE)

        h.text_enc_restriction = ExtendedTagHeader.RESTRICT_TEXT_ENC_UTF8
        assert (h.tag_size_restriction ==
                ExtendedTagHeader.RESTRICT_TAG_SZ_TINY)
        assert (h.text_enc_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_ENC_UTF8)
        assert (h.text_length_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_LEN_NONE)
        assert (h.image_enc_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_ENC_NONE)
        assert (h.image_size_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_SZ_NONE)

        h.text_length_restriction = ExtendedTagHeader.RESTRICT_TEXT_LEN_30
        assert (h.tag_size_restriction ==
                ExtendedTagHeader.RESTRICT_TAG_SZ_TINY)
        assert (h.text_enc_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_ENC_UTF8)
        assert (h.text_length_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_LEN_30)
        assert (h.image_enc_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_ENC_NONE)
        assert (h.image_size_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_SZ_NONE)

        h.image_enc_restriction = ExtendedTagHeader.RESTRICT_IMG_ENC_PNG_JPG
        assert (h.tag_size_restriction ==
                ExtendedTagHeader.RESTRICT_TAG_SZ_TINY)
        assert (h.text_enc_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_ENC_UTF8)
        assert (h.text_length_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_LEN_30)
        assert (h.image_enc_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_ENC_PNG_JPG)
        assert (h.image_size_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_SZ_NONE)

        h.image_size_restriction = ExtendedTagHeader.RESTRICT_IMG_SZ_256
        assert (h.tag_size_restriction ==
                ExtendedTagHeader.RESTRICT_TAG_SZ_TINY)
        assert (h.text_enc_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_ENC_UTF8)
        assert (h.text_length_restriction ==
                ExtendedTagHeader.RESTRICT_TEXT_LEN_30)
        assert (h.image_enc_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_ENC_PNG_JPG)
        assert (h.image_size_restriction ==
                ExtendedTagHeader.RESTRICT_IMG_SZ_256)

        assert " 32 frames " in h.tag_size_restriction_description
        assert " 4 KB " in h.tag_size_restriction_description
        h.tag_size_restriction = ExtendedTagHeader.RESTRICT_TAG_SZ_LARGE
        assert " 128 frames " in h.tag_size_restriction_description
        h.tag_size_restriction = ExtendedTagHeader.RESTRICT_TAG_SZ_MED
        assert " 64 frames " in h.tag_size_restriction_description
        h.tag_size_restriction = ExtendedTagHeader.RESTRICT_TAG_SZ_SMALL
        assert " 32 frames " in h.tag_size_restriction_description
        assert " 40 KB " in h.tag_size_restriction_description

        assert (" UTF-8" in h.text_enc_restriction_description)
        h.text_enc_restriction = ExtendedTagHeader.RESTRICT_TEXT_ENC_NONE
        assert ("None" == h.text_enc_restriction_description)

        assert " 30 " in h.text_length_restriction_description
        h.text_length_restriction = ExtendedTagHeader.RESTRICT_TEXT_LEN_NONE
        assert ("None" == h.text_length_restriction_description)
        h.text_length_restriction = ExtendedTagHeader.RESTRICT_TEXT_LEN_1024
        assert " 1024 " in h.text_length_restriction_description
        h.text_length_restriction = ExtendedTagHeader.RESTRICT_TEXT_LEN_128
        assert " 128 " in h.text_length_restriction_description

        assert " PNG " in h.image_enc_restriction_description
        h.image_enc_restriction = ExtendedTagHeader.RESTRICT_IMG_ENC_NONE
        assert ("None" == h.image_enc_restriction_description)

        assert " 256x256 " in h.image_size_restriction_description
        h.image_size_restriction = ExtendedTagHeader.RESTRICT_IMG_SZ_NONE
        assert ("None" == h.image_size_restriction_description)
        h.image_size_restriction = ExtendedTagHeader.RESTRICT_IMG_SZ_64
        assert (" 64x64 pixels or smaller" in
                h.image_size_restriction_description)
        h.image_size_restriction = ExtendedTagHeader.RESTRICT_IMG_SZ_64_EXACT
        assert "exactly 64x64 pixels" in h.image_size_restriction_description

    def testRender(self):
        version = (2, 4, 0)
        dummy_data = b"\xab" * 50
        dummy_padding_len = 1024

        h = ExtendedTagHeader()
        h.update_bit = 1
        h.crc_bit = 1
        h.tag_size_restriction = ExtendedTagHeader.RESTRICT_TAG_SZ_MED
        h.text_enc_restriction = ExtendedTagHeader.RESTRICT_TEXT_ENC_UTF8
        h.text_length_restriction = ExtendedTagHeader.RESTRICT_TEXT_LEN_128
        h.image_enc_restriction = ExtendedTagHeader.RESTRICT_IMG_ENC_PNG_JPG
        h.image_size_restriction = ExtendedTagHeader.RESTRICT_IMG_SZ_256
        header = h.render(version, dummy_data, dummy_padding_len)

        h2 = ExtendedTagHeader()
        h2.parse(BytesIO(header), version)
        assert (h2.update_bit)
        assert (h2.crc_bit)
        assert (h2.restrictions_bit)
        assert (h.crc == h2.crc)
        assert (h.tag_size_restriction == h2.tag_size_restriction)
        assert (h.text_enc_restriction == h2.text_enc_restriction)
        assert (h.text_length_restriction == h2.text_length_restriction)
        assert (h.image_enc_restriction == h2.image_enc_restriction)
        assert (h.image_size_restriction == h2.image_size_restriction)

        assert h2.render(version, dummy_data, dummy_padding_len) == header

        # version 2.3
        header_23 = h.render((2,3,0), dummy_data, dummy_padding_len)

        h3 = ExtendedTagHeader()
        h3.parse(BytesIO(header_23), (2,3,0))
        assert not(h3.update_bit)
        assert (h3.crc_bit)
        assert not(h3.restrictions_bit)
        assert (h.crc == h3.crc)
        assert (0 == h3.tag_size_restriction)
        assert (0 == h3.text_enc_restriction)
        assert (0 == h3.text_length_restriction)
        assert (0 == h3.image_enc_restriction)
        assert (0 == h3.image_size_restriction)

    def testRenderCrcPadding(self):
        version = (2, 4, 0)

        h = ExtendedTagHeader()
        h.crc_bit = 1
        header = h.render(version, b"\x01", 0)

        h2 = ExtendedTagHeader()
        h2.parse(BytesIO(header), version)
        assert h.crc == h2.crc

    def testInvalidFlagBits(self):
        for bad_flags in [b"\x00\x20", b"\x01\x01"]:
            h = ExtendedTagHeader()
            try:
                h.parse(BytesIO(b"\x00\x00\x00\xff" + bad_flags), (2, 4, 0))
            except TagException:
                pass
            else:
                assert not("Bad ExtendedTagHeader flags, expected "
                             "TagException")

class TestFrameHeader(unittest.TestCase):
    def testCtor(self):
        h = FrameHeader(b"TIT2", ID3_DEFAULT_VERSION)
        assert (h.size == 10)
        assert (h.id == b"TIT2")
        assert (h.data_size == 0)
        assert (h._flags == [0] * 16)

        h = FrameHeader(b"TIT2", (2, 3, 0))
        assert (h.size == 10)
        assert (h.id == b"TIT2")
        assert (h.data_size == 0)
        assert (h._flags == [0] * 16)

        h = FrameHeader(b"TIT2", (2, 2, 0))
        assert (h.size == 6)
        assert (h.id == b"TIT2")
        assert (h.data_size == 0)
        assert (h._flags == [0] * 16)

    def testBitMask(self):
        for v in [(2, 2, 0), (2, 3, 0)]:
            h = FrameHeader(b"TXXX", v)
            assert (h.TAG_ALTER == 0)
            assert (h.FILE_ALTER == 1)
            assert (h.READ_ONLY == 2)
            assert (h.COMPRESSED == 8)
            assert (h.ENCRYPTED == 9)
            assert (h.GROUPED == 10)
            assert (h.UNSYNC == 14)
            assert (h.DATA_LEN == 4)

        for v in [(2, 4, 0), (1, 0, 0), (1, 1, 0)]:
            h = FrameHeader(b"TXXX", v)
            assert (h.TAG_ALTER == 1)
            assert (h.FILE_ALTER == 2)
            assert (h.READ_ONLY == 3)
            assert (h.COMPRESSED == 12)
            assert (h.ENCRYPTED == 13)
            assert (h.GROUPED == 9)
            assert (h.UNSYNC == 14)
            assert (h.DATA_LEN == 15)

        for v in [(2, 5, 0), (3, 0, 0)]:
            try:
                h = FrameHeader(b"TIT2", v)
            except ValueError:
                pass
            else:
                assert not("Expected a ValueError from invalid version, "
                             "but got success")

        for v in [1, "yes", "no", True, 23]:
            h = FrameHeader(b"APIC", (2, 4, 0))
            h.tag_alter = v
            h.file_alter = v
            h.read_only = v
            h.compressed = v
            h.encrypted = v
            h.grouped = v
            h.unsync = v
            h.data_length_indicator = v
            assert (h.tag_alter == 1)
            assert (h.file_alter == 1)
            assert (h.read_only == 1)
            assert (h.compressed == 1)
            assert (h.encrypted == 1)
            assert (h.grouped == 1)
            assert (h.unsync == 1)
            assert (h.data_length_indicator == 1)

        for v in [0, False, None]:
            h = FrameHeader(b"APIC", (2, 4, 0))
            h.tag_alter = v
            h.file_alter = v
            h.read_only = v
            h.compressed = v
            h.encrypted = v
            h.grouped = v
            h.unsync = v
            h.data_length_indicator = v
            assert (h.tag_alter == 0)
            assert (h.file_alter == 0)
            assert (h.read_only == 0)
            assert (h.compressed == 0)
            assert (h.encrypted == 0)
            assert (h.grouped == 0)
            assert (h.unsync == 0)
            assert (h.data_length_indicator == 0)

        h1 = FrameHeader(b"APIC", (2, 3, 0))
        h1.tag_alter = True
        h1.grouped = True
        h1.file_alter = 1
        h1.encrypted = None
        h1.compressed = 4
        h1.data_length_indicator = 0
        h1.read_only = 1
        h1.unsync = 1

        h2 = FrameHeader(b"APIC", (2, 4, 0))
        assert (h2.tag_alter == 0)
        assert (h2.grouped == 0)
        h2.copyFlags(h1)
        assert (h2.tag_alter)
        assert (h2.grouped)
        assert (h2.file_alter)
        assert not(h2.encrypted)
        assert (h2.compressed)
        assert not(h2.data_length_indicator)
        assert (h2.read_only)
        assert (h2.unsync)

    def testValidFrameId(self):
        for id in [b"", b"a", b"tx", b"tit", b"TIT", b"Tit2", b"aPic"]:
            assert not(FrameHeader._isValidFrameId(id))
        for id in [b"TIT2", b"APIC", b"1234"]:
            assert FrameHeader._isValidFrameId(id)

    def testRenderWithUnsyncTrue(self):
        h = FrameHeader(b"TIT2", ID3_DEFAULT_VERSION)
        h.unsync = True
        with pytest.raises(NotImplementedError):
            h.render(100)



"""
Test functions and data by Jason Penney.
https://bitbucket.org/nicfit/eyed3/issue/32/mp3audioinfotime_secs-incorrect-for-mpeg2

To test individual files use:::

    python -m test.mp3.test_infos <file>
"""
import eyed3
import sys
import os
from decimal import Decimal
from .. import DATA_D, unittest


def _do_test(reported, expected):
    if reported != expected:
        return (False, "eyed3 reported %s (expected %s)" %
                (str(reported), str(expected)))
    return (True, '')


def _translate_mode(mode):
    if mode == 'simple':
        return 'Stereo'
    if mode == 'mono':
        return 'Mono'
    if mode == 'joint' or mode == 'force':
        return 'Joint stereo'
    if mode == 'dual-mono':
        return 'Dual channel stereo'
    raise RuntimeError("unknown mode: %s" % mode)


def _test_file(pth):
    errors = []
    info = os.path.splitext(os.path.basename(pth))[0].split(' ')
    fil = eyed3.load(pth)

    tests = [
        ('mpeg_version', Decimal(str(fil.info.mp3_header.version)),
          Decimal(info[0][-3:])),
        ('sample_freq', Decimal(str(fil.info.mp3_header.sample_freq))/1000,
         Decimal(info[1][:-3])),
        ('vbr', fil.info.bit_rate[0], bool(info[2] == '__vbr__')),
        ('stereo_mode', fil.info.mode, _translate_mode(info[3])),
        ('duration', round(fil.info.time_secs), 10),

    ]

    if info[2] != '__vbr__':
        tests.append(('bit_rate', fil.info.bit_rate[1], int(info[2][:-4])))

    for test, reported, expected in tests:
        (passed, msg) = _do_test(reported, expected)
        if not passed:
            errors.append("%s: %s" % (test, msg))

    print("%s: %s" % (os.path.basename(pth), 'FAIL' if errors else 'ok'))
    for err in errors:
        print("    %s" % err)

    return errors


@unittest.skipIf(not os.path.exists(DATA_D), "test requires data files")
def test_mp3_infos(do_assert=True):
    data_d = os.path.join(DATA_D, "mp3_samples")
    mp3s = sorted([f for f in os.listdir(data_d) if f.endswith(".mp3")])

    for mp3_file in mp3s:
        errors = _test_file(os.path.join(data_d, mp3_file))
        if do_assert:
            assert(len(errors) == 0)

if __name__ == "__main__":
    if len(sys.argv) < 2:
        test_mp3_infos(do_assert=False)
    else:
        for mp3_file in sys.argv[1:]:
            errors = _test_file(mp3_file)



import os
import shutil
import tempfile
import unittest
from pathlib import Path

import pytest
import eyed3
from eyed3 import main, id3, core, utils
from . import DATA_D, RedirectStdStreams


def testPluginOption():
    for arg in ["--help", "-h"]:
        # When help is requested and no plugin is specified, use default
        with RedirectStdStreams() as out:
            try:
                args, _, config = main.parseCommandLine([arg])
            except SystemExit as ex:
                assert ex.code == 0
                out.stdout.seek(0)
                sout = out.stdout.read()
                assert sout.find("Plugin options:\n  Classic eyeD3") != -1

    # When help is requested and all default plugin names are specified
    for plugin_name in ["classic"]:
        for args in [["--plugin=%s" % plugin_name, "--help"]]:
            with RedirectStdStreams() as out:
                try:
                    args, _, config = main.parseCommandLine(args)
                except SystemExit as ex:
                    assert ex.code == 0
                    out.stdout.seek(0)
                    sout = out.stdout.read()
                    assert sout.find("Plugin options:\n  Classic eyeD3") != -1


@unittest.skipIf(not Path(DATA_D).exists(), "test requires data files")
def testReadEmptyMp3():
    with RedirectStdStreams() as out:
        args, _, config = main.parseCommandLine([os.path.join(DATA_D,
                                                              "test.mp3")])
        retval = main.main(args, config)
        assert retval == 0
    assert out.stderr.read().find("No ID3 v1.x/v2.x tag found") != -1


class TestDefaultPlugin(unittest.TestCase):
    def __init__(self, name):
        super(TestDefaultPlugin, self).__init__(name)
        self.orig_test_file = "%s/test.mp3" % DATA_D
        self.test_file = "/tmp/test.mp3"
        fd, self.test_file = tempfile.mkstemp(suffix=".mp3")
        os.close(fd)

    @unittest.skipIf(not os.path.exists(DATA_D), "test requires data files")
    def setUp(self):
        shutil.copy(self.orig_test_file, self.test_file)

    def tearDown(self):
        # TODO: could remove the tag and compare audio file to original
        os.remove(self.test_file)

    @staticmethod
    def _addVersionOpt(version, opts):
        if version == id3.ID3_DEFAULT_VERSION:
            return

        if version[0] == 1:
            opts.append("--to-v1.1")
        elif version[:2] == (2, 3):
            opts.append("--to-v2.3")
        elif version[:2] == (2, 4):
            opts.append("--to-v2.4")
        else:
            assert not "Unhandled version"

    def testNewTagArtist(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["-a", "The Cramps", self.test_file],
                      ["--artist=The Cramps", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert retval == 0

            af = eyed3.load(self.test_file)
            assert af is not None
            assert af.tag is not None
            assert af.tag.artist == "The Cramps"

    def testNewTagComposer(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["--composer=H.R.", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert retval == 0

            af = eyed3.load(self.test_file)
            assert  af is not None
            assert  af.tag is not None
            assert af.tag.composer == "H.R."

    def testNewTagAlbum(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["-A", "Psychedelic Jungle", self.test_file],
                      ["--album=Psychedelic Jungle", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.album == "Psychedelic Jungle")

    def testNewTagAlbumArtist(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["-b", "Various Artists", self.test_file],
                      ["--album-artist=Various Artists", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert af is not None
            assert af.tag is not None
            assert af.tag.album_artist == "Various Artists"

    def testNewTagTitle(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["-t", "Green Door", self.test_file],
                      ["--title=Green Door", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.title == "Green Door")

    def testNewTagTrackNum(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["-n", "14", self.test_file],
                      ["--track=14", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.track_num[0] == 14)

    def testNewTagTrackNumInvalid(self):
        for opts in [ ["-n", "abc", self.test_file],
                      ["--track=-14", self.test_file]
                      ]:

            with RedirectStdStreams() as out:
                try:
                    args, _, config = main.parseCommandLine(opts)
                except SystemExit as ex:
                    assert ex.code != 0
                else:
                    assert not("Should not have gotten here")

    def testNewTagTrackTotal(self, version=id3.ID3_DEFAULT_VERSION):
        if version[0] == 1:
            # No support for this in v1.x
            return

        for opts in [ ["-N", "14", self.test_file],
                      ["--track-total=14", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.track_num[1] == 14)

    def testNewTagGenre(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["-G", "Rock", self.test_file],
                      ["--genre=Rock", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.genre.name == "Rock")
            assert (af.tag.genre.id == 17)

    def testNewTagNonStdGenre(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in (("-G", "108", "--non-std-genre", self.test_file),
                     ("--genre=108", "--non-std-genre", self.test_file)):
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert retval == 0

            af = eyed3.load(self.test_file)
            assert af.tag.non_std_genre.name == "108"
            assert af.tag.non_std_genre.id is None

    def testNewTagYear(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["-Y", "1981", self.test_file],
                      ["--release-year=1981", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            if version == id3.ID3_V2_3:
                assert (af.tag.original_release_date.year == 1981)
            else:
                assert (af.tag.release_date.year == 1981)

    def testNewTagReleaseDate(self, version=id3.ID3_DEFAULT_VERSION):
        for date in ["1981", "1981-03-06", "1981-03"]:
            orig_date = core.Date.parse(date)

            for opts in [ ["--release-date=%s" % str(date), self.test_file] ]:
                self._addVersionOpt(version, opts)

                with RedirectStdStreams() as out:
                    args, _, config = main.parseCommandLine(opts)
                    retval = main.main(args, config)
                    assert (retval == 0)

                af = eyed3.load(self.test_file)
                assert (af is not None)
                assert (af.tag is not None)
                assert (af.tag.release_date == orig_date)

    def testNewTagOrigRelease(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["--orig-release-date=1981", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.original_release_date.year == 1981)

    def testNewTagRecordingDate(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["--recording-date=1993-10-30", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.recording_date.year == 1993)
            assert (af.tag.recording_date.month == 10)
            assert (af.tag.recording_date.day == 30)

    def testNewTagEncodingDate(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["--encoding-date=2012-10-23T20:22", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.encoding_date.year == 2012)
            assert (af.tag.encoding_date.month == 10)
            assert (af.tag.encoding_date.day == 23)
            assert (af.tag.encoding_date.hour == 20)
            assert (af.tag.encoding_date.minute == 22)

    def testNewTagTaggingDate(self, version=id3.ID3_DEFAULT_VERSION):
        for opts in [ ["--tagging-date=2012-10-23T20:22", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.tagging_date.year == 2012)
            assert (af.tag.tagging_date.month == 10)
            assert (af.tag.tagging_date.day == 23)
            assert (af.tag.tagging_date.hour == 20)
            assert (af.tag.tagging_date.minute == 22)

    def testNewTagPlayCount(self):
        for expected, opts in [ (0, ["--play-count=0", self.test_file]),
                                (1, ["--play-count=+1", self.test_file]),
                                (6, ["--play-count=+5", self.test_file]),
                                (7, ["--play-count=7", self.test_file]),
                                (10000, ["--play-count=10000", self.test_file]),
                              ]:

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.play_count == expected)

    def testNewTagPlayCountInvalid(self):
        for expected, opts in [ (0, ["--play-count=", self.test_file]),
                                (0, ["--play-count=-24", self.test_file]),
                                (0, ["--play-count=+", self.test_file]),
                                (0, ["--play-count=abc", self.test_file]),
                                (0, ["--play-count=False", self.test_file]),
                              ]:

            with RedirectStdStreams() as out:
                try:
                    args, _, config = main.parseCommandLine(opts)
                except SystemExit as ex:
                    assert ex.code != 0
                else:
                    assert not("Should not have gotten here")

    def testNewTagBpm(self):
        for expected, opts in [ (1, ["--bpm=1", self.test_file]),
                                (180, ["--bpm=180", self.test_file]),
                                (117, ["--bpm", "116.7", self.test_file]),
                                (116, ["--bpm", "116.4", self.test_file]),
                              ]:

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.bpm == expected)

    def testNewTagBpmInvalid(self):
        for expected, opts in [ (0, ["--bpm=", self.test_file]),
                                (0, ["--bpm=-24", self.test_file]),
                                (0, ["--bpm=+", self.test_file]),
                                (0, ["--bpm=abc", self.test_file]),
                                (0, ["--bpm", "=180", self.test_file]),
                              ]:

            with RedirectStdStreams() as out:
                try:
                    args, _, config = main.parseCommandLine(opts)
                except SystemExit as ex:
                    assert ex.code != 0
                else:
                    assert not("Should not have gotten here")

    def testNewTagPublisher(self):
        for expected, opts in [
                ("SST", ["--publisher", "SST", self.test_file]),
                ("Dischord", ["--publisher=Dischord", self.test_file]),
               ]:

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.publisher == expected)

    def testUniqueFileId_1(self):
        with RedirectStdStreams() as out:
            assert out
            args, _, config = main.parseCommandLine(["--unique-file-id", "Travis:Me",
                                                     self.test_file])
            retval = main.main(args, config)
            assert retval == 0

        af = eyed3.load(self.test_file)
        assert len(af.tag.unique_file_ids) == 1
        assert af.tag.unique_file_ids.get(b"Travis").uniq_id == b"Me"

    def testUniqueFileId_dup(self):
        with RedirectStdStreams() as out:
            assert out
            args, _, config = \
                main.parseCommandLine(["--unique-file-id", "Travis:Me",
                                       "--unique-file-id=Travis:Me",
                                       self.test_file])
            retval = main.main(args, config)
            assert retval == 0

        af = eyed3.load(self.test_file)
        assert len(af.tag.unique_file_ids) == 1
        assert af.tag.unique_file_ids.get(b"Travis").uniq_id == b"Me"

    def testUniqueFileId_N(self):
        # Add 3
        with RedirectStdStreams() as out:
            assert out
            args, _, config = \
                main.parseCommandLine(["--unique-file-id", "Travis:Me",
                                       "--unique-file-id=Engine:Kid",
                                       "--unique-file-id", "Owner:Kid",
                                       self.test_file])
            retval = main.main(args, config)
            assert retval == 0

        af = eyed3.load(self.test_file)
        assert len(af.tag.unique_file_ids) == 3
        assert af.tag.unique_file_ids.get("Travis").uniq_id == b"Me"
        assert af.tag.unique_file_ids.get("Engine").uniq_id == b"Kid"
        assert af.tag.unique_file_ids.get(b"Owner").uniq_id == b"Kid"

        # Remove 2
        with RedirectStdStreams() as out:
            assert out
            args, _, config = \
                main.parseCommandLine(["--unique-file-id", "Travis:",
                                       "--unique-file-id=Engine:",
                                       "--unique-file-id", "Owner:Kid",
                                       self.test_file])
            retval = main.main(args, config)
            assert retval == 0

        af = eyed3.load(self.test_file)
        assert len(af.tag.unique_file_ids) == 1

        # Remove not found ID
        with RedirectStdStreams() as out:
            args, _, config = \
                main.parseCommandLine(["--unique-file-id", "Travis:",
                                       self.test_file])
            retval = main.main(args, config)
            assert retval == 0

        sout = out.stdout.read()
        assert "Unique file ID 'Travis' not found" in sout

        af = eyed3.load(self.test_file)
        assert len(af.tag.unique_file_ids) == 1

    # TODO:
    #       --text-frame, --user-text-frame
    #       --url-frame, --user-user-frame
    #       --add-image, --remove-image, --remove-all-images, --write-images
    #       etc.
    #       --rename, --force-update, -1, -2, --exclude

    def testNewTagSimpleComment(self, version=id3.ID3_DEFAULT_VERSION):
        if version[0] == 1:
            # No support for this in v1.x
            return

        for opts in [ ["-c", "Starlette", self.test_file],
                      ["--comment=Starlette", self.test_file] ]:
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)
            assert (af.tag.comments[0].text == "Starlette")
            assert (af.tag.comments[0].description == "")

    def testAddRemoveComment(self, version=id3.ID3_DEFAULT_VERSION):
        if version[0] == 1:
            # No support for this in v1.x
            return

        comment = "Why can't I be you?"
        for i, (c, d, l) in enumerate([(comment, "c0", None),
                                       (comment, "c1", None),
                                       (comment, "c2", 'eng'),
                                       ("¿Por qué no puedo ser tú ?", "c2",
                                        'esp'),
                                      ]):

            darg = ":{}".format(d) if d else ""
            larg = ":{}".format(l) if l else ""
            opts = ["--add-comment={c}{darg}{larg}".format(**locals()),
                    self.test_file]

            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)

            tag_comment = af.tag.comments.get(d or "",
                                              lang=utils.b(l if l else "eng"))
            assert (tag_comment.text == c)
            assert (tag_comment.description == d or "")
            assert (tag_comment.lang == utils.b(l if l else "eng"))

        for d, l in [("c0", None),
                     ("c1", None),
                     ("c2", "eng"),
                     ("c2", "esp"),
                    ]:

            larg = ":{}".format(l) if l else ""
            opts = ["--remove-comment={d}{larg}".format(**locals()),
                    self.test_file]
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            tag_comment = af.tag.comments.get(d,
                                              lang=utils.b(l if l else "eng"))
            assert tag_comment is None

        assert (len(af.tag.comments) == 0)

    def testRemoveAllComments(self, version=id3.ID3_DEFAULT_VERSION):
        if version[0] == 1:
            # No support for this in v1.x
            return

        comment = "Why can't I be you?"
        for i, (c, d, l) in enumerate([(comment, "c0", None),
                                       (comment, "c1", None),
                                       (comment, "c2", 'eng'),
                                       ("¿Por qué no puedo ser tú ?", "c2",
                                        'esp'),
                                       (comment, "c4", "ger"),
                                       (comment, "c4", "rus"),
                                       (comment, "c5", "rus"),
                                      ]):

            darg = ":{}".format(d) if d else ""
            larg = ":{}".format(l) if l else ""
            opts = ["--add-comment={c}{darg}{larg}".format(**locals()),
                    self.test_file]

            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)

            tag_comment = af.tag.comments.get(d or "",
                                              lang=utils.b(l if l else "eng"))
            assert (tag_comment.text == c)
            assert (tag_comment.description == d or "")
            assert (tag_comment.lang == utils.b(l if l else "eng"))

        opts = ["--remove-all-comments", self.test_file]
        self._addVersionOpt(version, opts)

        with RedirectStdStreams() as out:
            args, _, config = main.parseCommandLine(opts)
            retval = main.main(args, config)
            assert (retval == 0)

        af = eyed3.load(self.test_file)
        assert (len(af.tag.comments) == 0)

    def testAddRemoveLyrics(self, version=id3.ID3_DEFAULT_VERSION):
        if version[0] == 1:
            # No support for this in v1.x
            return

        comment = "Why can't I be you?"
        for i, (c, d, l) in enumerate([(comment, "c0", None),
                                       (comment, "c1", None),
                                       (comment, "c2", 'eng'),
                                       ("¿Por qué no puedo ser tú ?", "c2",
                                        'esp'),
                                      ]):

            darg = ":{}".format(d) if d else ""
            larg = ":{}".format(l) if l else ""
            opts = ["--add-comment={c}{darg}{larg}".format(**locals()),
                    self.test_file]

            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            assert (af is not None)
            assert (af.tag is not None)

            tag_comment = af.tag.comments.get(d or "",
                                              lang=utils.b(l if l else "eng"))
            assert (tag_comment.text == c)
            assert (tag_comment.description == d or "")
            assert (tag_comment.lang == utils.b(l if l else "eng"))

        for d, l in [("c0", None),
                     ("c1", None),
                     ("c2", "eng"),
                     ("c2", "esp"),
                    ]:

            larg = ":{}".format(l) if l else ""
            opts = ["--remove-comment={d}{larg}".format(**locals()),
                    self.test_file]
            self._addVersionOpt(version, opts)

            with RedirectStdStreams() as out:
                args, _, config = main.parseCommandLine(opts)
                retval = main.main(args, config)
                assert (retval == 0)

            af = eyed3.load(self.test_file)
            tag_comment = af.tag.comments.get(d,
                                              lang=utils.b(l if l else "eng"))
            assert tag_comment is None

        assert (len(af.tag.comments) == 0)

    def testNewTagAll(self, version=id3.ID3_DEFAULT_VERSION):
        self.testNewTagArtist(version)
        self.testNewTagAlbum(version)
        self.testNewTagTitle(version)
        self.testNewTagTrackNum(version)
        self.testNewTagTrackTotal(version)
        self.testNewTagGenre(version)
        self.testNewTagYear(version)
        self.testNewTagSimpleComment(version)

        af = eyed3.load(self.test_file)
        assert (af.tag.artist == "The Cramps")
        assert (af.tag.album == "Psychedelic Jungle")
        assert (af.tag.title == "Green Door")
        assert (af.tag.track_num == (14, 14 if version[0] != 1 else None))
        assert ((af.tag.genre.name, af.tag.genre.id) == ("Rock", 17))
        if version == id3.ID3_V2_3:
            assert (af.tag.original_release_date.year == 1981)
        else:
            assert (af.tag.release_date.year == 1981)

        if version[0] != 1:
            assert (af.tag.comments[0].text == "Starlette")
            assert (af.tag.comments[0].description == "")

        assert (af.tag.version == version)

    def testNewTagAllVersion1(self):
        self.testNewTagAll(version=id3.ID3_V1_1)

    def testNewTagAllVersion2_3(self):
        self.testNewTagAll(version=id3.ID3_V2_3)

    def testNewTagAllVersion2_4(self):
        self.testNewTagAll(version=id3.ID3_V2_4)


## XXX: newer pytest test below.


@pytest.mark.skipif(not Path(DATA_D).exists(), reason="test requires data files")
def test_lyrics(audiofile, tmpdir, eyeD3):
    lyrics_files = []
    for i in range(1, 4):
        lfile = tmpdir / "lryics{:d}".format(i)
        lfile.write_text((str(i) * (100 * i)), "utf8")
        lyrics_files.append(lfile)

    audiofile = eyeD3(audiofile,
                      ["--add-lyrics", "{}".format(lyrics_files[0]),
                        "--add-lyrics", "{}:desc".format(lyrics_files[1]),
                        "--add-lyrics", "{}:foo:en".format(lyrics_files[1]),
                        "--add-lyrics", "{}:foo:es".format(lyrics_files[2]),
                        "--add-lyrics", "{}:foo:de".format(lyrics_files[0]),
                       ])
    assert len(audiofile.tag.lyrics) == 5
    assert audiofile.tag.lyrics.get("").text == ("1" * 100)
    assert audiofile.tag.lyrics.get("desc").text == ("2" * 200)
    assert audiofile.tag.lyrics.get("foo", "en").text == ("2" * 200)
    assert audiofile.tag.lyrics.get("foo", "es").text == ("3" * 300)
    assert audiofile.tag.lyrics.get("foo", "de").text == ("1" * 100)

    audiofile = eyeD3(audiofile, ["--remove-lyrics", "foo:xxx"])
    assert len(audiofile.tag.lyrics) == 5

    audiofile = eyeD3(audiofile, ["--remove-lyrics", "foo:es"])
    assert len(audiofile.tag.lyrics) == 4

    audiofile = eyeD3(audiofile, ["--remove-lyrics", "desc"])
    assert len(audiofile.tag.lyrics) == 3

    audiofile = eyeD3(audiofile, ["--remove-all-lyrics"])
    assert len(audiofile.tag.lyrics) == 0

    eyeD3(audiofile, ["--add-lyrics", "eminem.txt"], expected_retval=2)


@pytest.mark.skipif(not Path(DATA_D).exists(), reason="test requires data files")
def test_all(audiofile, image, eyeD3):
    audiofile = eyeD3(audiofile,
                      ["--artist", "Cibo Matto",
                       "--album-artist", "Cibo Matto",
                       "--album", "Viva! La Woman",
                       "--title", "Apple",
                       "--track=1", "--track-total=11",
                       "--disc-num=1", "--disc-total=1",
                       "--genre", "Pop",
                       "--release-date=1996-01-16",
                       "--orig-release-date=1996-01-16",
                       "--recording-date=1995-01-16",
                       "--encoding-date=1999-01-16",
                       "--tagging-date=1999-01-16",
                       "--comment", "From Japan",
                       "--publisher=\'Warner Brothers\'",
                       "--play-count=666",
                       "--bpm=99",
                       "--unique-file-id", "mishmash:777abc",
                       "--add-comment", "Trip Hop",
                       "--add-comment", "Quirky:Mood",
                       "--add-comment", "Kimyōna:Mood:jp",
                       "--add-comment", "Test:XXX",
                       "--add-popularity", "travis@ppbox.com:212:999",
                       "--fs-encoding=latin1",
                       "--no-config",
                       "--add-object", "{}:image/gif".format(image),
                       "--composer", "Cibo Matto",
                       ])
    assert audiofile


@pytest.mark.skipif(not Path(DATA_D).exists(), reason="test requires data files")
def test_removeTag_v1(audiofile, eyeD3):
    assert audiofile.tag is None
    audiofile = eyeD3(audiofile, ["-1", "-a", "Government Issue"])
    assert audiofile.tag.version == id3.ID3_V1_0
    audiofile = eyeD3(audiofile, ["--remove-v1"])
    assert audiofile.tag is None


@pytest.mark.skipif(not Path(DATA_D).exists(), reason="test requires data files")
def test_removeTag_v2(audiofile, eyeD3):
    assert audiofile.tag is None
    audiofile = eyeD3(audiofile, ["-2", "-a", "Integrity"])
    assert audiofile.tag.version == id3.ID3_V2_4
    audiofile = eyeD3(audiofile, ["--remove-v2"])
    assert audiofile.tag is None


@pytest.mark.skipif(not Path(DATA_D).exists(), reason="test requires data files")
def test_removeTagWithBoth_v1(audiofile, eyeD3):
    audiofile = eyeD3(eyeD3(audiofile, ["-1", "-a", "Face Value"]),
                      ["-2", "-a", "Poison Idea"])
    v1_view = eyeD3(audiofile, ["-1"], reload_version=id3.ID3_V1)
    v2_view = eyeD3(audiofile, ["-2"], reload_version=id3.ID3_V2)
    assert audiofile.tag.version == id3.ID3_V2_4
    assert v1_view.tag.version == id3.ID3_V1_0
    assert v2_view.tag.version == id3.ID3_V2_4
    audiofile = eyeD3(audiofile, ["--remove-v1"])
    assert audiofile.tag.version == id3.ID3_V2_4
    assert eyeD3(audiofile, ["-1"], reload_version=id3.ID3_V1).tag is None
    v2_tag = eyeD3(audiofile, ["-2"], reload_version=id3.ID3_V2).tag
    assert v2_tag is not None
    assert v2_tag.artist == "Poison Idea"


@pytest.mark.skipif(not Path(DATA_D).exists(), reason="test requires data files")
def test_removeTagWithBoth_v2(audiofile, eyeD3):
    audiofile = eyeD3(eyeD3(audiofile, ["-1", "-a", "Face Value"]),
                      ["-2", "-a", "Poison Idea"])
    v1_view = eyeD3(audiofile, ["-1"], reload_version=id3.ID3_V1)
    v2_view = eyeD3(audiofile, ["-2"], reload_version=id3.ID3_V2)
    assert audiofile.tag.version == id3.ID3_V2_4
    assert v1_view.tag.version == id3.ID3_V1_0
    assert v2_view.tag.version == id3.ID3_V2_4
    audiofile = eyeD3(audiofile, ["--remove-v2"])
    assert audiofile.tag.version == id3.ID3_V1_0
    assert eyeD3(audiofile, ["-2"], reload_version=id3.ID3_V2).tag is None
    v1_tag = eyeD3(audiofile, ["-1"], reload_version=id3.ID3_V1).tag
    assert v1_tag is not None and v1_tag.artist == "Face Value"


@pytest.mark.skipif(not Path(DATA_D).exists(), reason="test requires data files")
def test_removeTagWithBoth_v2_withConvert(audiofile, eyeD3):
    audiofile = eyeD3(eyeD3(audiofile, ["-1", "-a", "Face Value"]),
                      ["-2", "-a", "Poison Idea"])
    v1_view = eyeD3(audiofile, ["-1"], reload_version=id3.ID3_V1)
    v2_view = eyeD3(audiofile, ["-2"], reload_version=id3.ID3_V2)
    assert audiofile.tag.version == id3.ID3_V2_4
    assert v1_view.tag.version == id3.ID3_V1_0
    assert v2_view.tag.version == id3.ID3_V2_4
    audiofile = eyeD3(audiofile, ["--remove-v2", "--to-v1"])
    assert audiofile.tag.version == id3.ID3_V1_0
    assert eyeD3(audiofile, ["-2"], reload_version=id3.ID3_V2).tag is None
    v1_tag = eyeD3(audiofile, ["-1"], reload_version=id3.ID3_V1).tag
    assert v1_tag is not None and v1_tag.artist == "Face Value"


@pytest.mark.skipif(not Path(DATA_D).exists(), reason="test requires data files")
def test_removeTagWithBoth_v1_withConvert(audiofile, eyeD3):
    audiofile = eyeD3(eyeD3(audiofile, ["-1", "-a", "Face Value"]),
                      ["-2", "-a", "Poison Idea"])
    v1_view = eyeD3(audiofile, ["-1"], reload_version=id3.ID3_V1)
    v2_view = eyeD3(audiofile, ["-2"], reload_version=id3.ID3_V2)
    assert audiofile.tag.version == id3.ID3_V2_4
    assert v1_view.tag.version == id3.ID3_V1_0
    assert v2_view.tag.version == id3.ID3_V2_4
    audiofile = eyeD3(audiofile, ["--remove-v1", "--to-v2.3"])
    assert audiofile.tag.version == id3.ID3_V2_3
    assert eyeD3(audiofile, ["-1"], reload_version=id3.ID3_V1).tag is None
    v2_tag = eyeD3(audiofile, ["-2"], reload_version=id3.ID3_V2).tag
    assert v2_tag is not None and v2_tag.artist == "Poison Idea"


def test_clearGenre(audiofile, eyeD3):
    audiofile = eyeD3(audiofile, ["--genre=Rock"])
    assert audiofile.tag.genre.name, audiofile.tag.genre.name == ("Rock", 17)
    audiofile = eyeD3(audiofile, ["--genre", ""])
    assert audiofile.tag.genre is None



import dataclasses
import pytest
from pytest import approx
from eyed3.id3 import ID3_V2_3, ID3_V2_4, ID3_V2_2
from eyed3.id3.frames import RelVolAdjFrameV23, RelVolAdjFrameV24, FrameException, FrameHeader


def test_default_v23():
    f = RelVolAdjFrameV23()
    assert f.id == b"RVAD"

    f.adjustments = RelVolAdjFrameV23.VolumeAdjustments()
    f.render()

    f2 = RelVolAdjFrameV23()
    f2.parse(f.data, f.header)

    assert f.adjustments == f2.adjustments
    assert set(dataclasses.astuple(f.adjustments)) == {0}


def test_RelVolAdjFrameV23_invalid_version():
    f = RelVolAdjFrameV23()
    f.adjustments = RelVolAdjFrameV23.VolumeAdjustments()
    f.render()

    f.header = FrameHeader(f.id, ID3_V2_4)
    with pytest.raises(FrameException):
        f.parse(f.data, f.header)


def test_RelVolAdjFrameV23_outofbounds():
    f = RelVolAdjFrameV23()
    f.adjustments = RelVolAdjFrameV23.VolumeAdjustments()
    with pytest.raises(ValueError):
        f.adjustments.front_right = 2**16 + 1
        f.render()
    with pytest.raises(ValueError):
        f.adjustments.front_right = -(2**16) - 1
        f.render()

    f.adjustments.front_right = 2**16
    assert f.render()

    f2 = RelVolAdjFrameV23()
    data = bytearray(f.data)
    data[1] = 32
    f.data = bytes(data)
    with pytest.raises(FrameException):
        f2.parse(f.data, f.header)


def test_v23_supported():
    f = RelVolAdjFrameV23()
    f.adjustments = RelVolAdjFrameV23.VolumeAdjustments(
        front_right=-10, front_left=2, front_right_peak=15, front_left_peak=15,
        back_right=54, back_left=-24, back_right_peak=100, back_left_peak=101,
        front_center=10, front_center_peak=15,
        bass=-666, bass_peak=5000,
    )
    assert f.adjustments.has_front_channel
    assert f.adjustments.has_back_channel
    assert f.adjustments.has_front_channel
    assert f.adjustments.has_bass_channel
    assert not f.adjustments.has_master_channel
    assert not f.adjustments.has_other_channel
    assert not f.adjustments.has_back_center_channel

    f.render()

    f2 = RelVolAdjFrameV23()
    f2.parse(f.data, f.header)

    assert f.adjustments == f2.adjustments


def test_v23_unsupported():
    f = RelVolAdjFrameV23()
    f.adjustments = RelVolAdjFrameV23.VolumeAdjustments(
        master=999, master_peak=999, other=333, other_peak=333, back_center=-5, back_center_peak=1,
        front_right=10, front_left=-2, front_right_peak=15, front_left_peak=15,
        back_right=-54, back_left=-24, back_right_peak=100, back_left_peak=101,
        front_center=10, front_center_peak=15,
        bass=666, bass_peak=5000,
    )
    assert f.adjustments.has_front_channel
    assert f.adjustments.has_back_channel
    assert f.adjustments.has_front_channel
    assert f.adjustments.has_bass_channel
    assert f.adjustments.has_master_channel
    assert f.adjustments.has_other_channel
    assert f.adjustments.has_back_center_channel

    f.render()

    f2 = RelVolAdjFrameV23()
    f2.parse(f.data, f.header)

    assert f2.adjustments.has_front_channel
    assert f2.adjustments.has_back_channel
    assert f2.adjustments.has_front_channel
    assert f2.adjustments.has_bass_channel
    assert not f2.adjustments.has_master_channel
    assert not f2.adjustments.has_other_channel
    assert not f2.adjustments.has_back_center_channel

    f.adjustments.master = f.adjustments.master_peak = 0
    f.adjustments.other = f.adjustments.other_peak = 0
    f.adjustments.back_center = f.adjustments.back_center_peak = 0
    assert f.adjustments == f2.adjustments


def test_v23_bounds():
    f = RelVolAdjFrameV23()
    adjustments = dataclasses.asdict(RelVolAdjFrameV23.VolumeAdjustments())

    for a in adjustments.keys():
        values = dict(adjustments)
        for value, raises in [
            (65537, True), (-65537, True),
            (65536, False), (-65536, False),
            (32769, False), (-32768, False),
            (777, False), (-999, False),
            (0, False), (-0, False),
        ]:
            values[a] = value
            if raises:
                with pytest.raises(ValueError):
                    f.adjustments = RelVolAdjFrameV23.VolumeAdjustments(**values)
                    f.render()
            else:
                f.adjustments = RelVolAdjFrameV23.VolumeAdjustments(**values)
                f.render()

        assert dataclasses.asdict(f.adjustments)[a] == value


def test_v23_optionals():
    f = RelVolAdjFrameV23()
    f.adjustments = RelVolAdjFrameV23.VolumeAdjustments(
        front_right=-10, front_left=2, front_right_peak=0, front_left_peak=0,
    )
    f.render()
    assert len(f.data) == 10

    f = RelVolAdjFrameV23()
    f.adjustments = RelVolAdjFrameV23.VolumeAdjustments(
        front_right=-10, front_left=2, front_right_peak=0, front_left_peak=0,
        bass=-666, bass_peak=5000,
    )
    f.render()
    assert len(f.data) == 26

    f = RelVolAdjFrameV23()
    f.adjustments = RelVolAdjFrameV23.VolumeAdjustments(
        front_right=-10, front_left=2, front_right_peak=0, front_left_peak=0,
        back_right=54, back_left=-24, back_right_peak=100, back_left_peak=101,
        front_center=10, front_center_peak=15,
    )
    f.render()
    assert len(f.data) == 22

    f = RelVolAdjFrameV23()
    f.adjustments = RelVolAdjFrameV23.VolumeAdjustments(
        front_right=-10, front_left=2, front_right_peak=0, front_left_peak=0,
        back_right=54, back_left=-24, back_right_peak=100, back_left_peak=101,
    )
    f.render()
    assert len(f.data) == 18


def test_default_v24():
    f = RelVolAdjFrameV24()
    assert f.id == b"RVA2"

    f.channel_type = RelVolAdjFrameV24.CHANNEL_TYPE_MASTER
    f.adjustment = -6.3
    f.peak = 666
    f.render()

    f2 = RelVolAdjFrameV24()
    f2.parse(f.data, f.header)
    assert f.adjustment == approx(-6.3)
    assert f2.peak == 666


def test_RVAD_RVA2(audiofile):
    # RVAD -> *RVA2
    audiofile.initTag(version=ID3_V2_3)
    audiofile.tag.frame_set[b"RVAD"] = RelVolAdjFrameV23()
    assert audiofile.tag.frame_set[b"RVAD"][0].adjustments is None
    adj = RelVolAdjFrameV23.VolumeAdjustments(front_left=20, front_right=19,
                                              back_left=-1, bass_peak=1024)
    audiofile.tag.frame_set[b"RVAD"][0].adjustments = adj

    # Convert to RVA2
    audiofile.tag.version = ID3_V2_4
    rva2_frames = {frame.channel_type: frame for frame in audiofile.tag.frame_set[b"RVA2"]}
    assert len(rva2_frames) == 4
    assert set(rva2_frames.keys()) == {RelVolAdjFrameV24.CHANNEL_TYPE_FRONT_LEFT,
                                       RelVolAdjFrameV24.CHANNEL_TYPE_FRONT_RIGHT,
                                       RelVolAdjFrameV24.CHANNEL_TYPE_BACK_LEFT,
                                       RelVolAdjFrameV24.CHANNEL_TYPE_BASS}
    assert rva2_frames[RelVolAdjFrameV24.CHANNEL_TYPE_FRONT_RIGHT].adjustment == approx(0.037109375)
    assert rva2_frames[RelVolAdjFrameV24.CHANNEL_TYPE_FRONT_LEFT].adjustment == approx(0.0390625)
    assert rva2_frames[RelVolAdjFrameV24.CHANNEL_TYPE_BACK_LEFT].adjustment == approx(-0.001953125)
    assert rva2_frames[RelVolAdjFrameV24.CHANNEL_TYPE_BASS].adjustment == 0
    assert rva2_frames[RelVolAdjFrameV24.CHANNEL_TYPE_BASS].peak == 1024

    # RVA2 --> RVAD
    audiofile.initTag(version=ID3_V2_4)
    assert len(audiofile.tag.frame_set) == 0
    for frame in rva2_frames.values():
        if b"RVA2" not in audiofile.tag.frame_set:
            audiofile.tag.frame_set[b"RVA2"] = frame
        else:
            audiofile.tag.frame_set[b"RVA2"].append(frame)
    assert len(audiofile.tag.frame_set) == 1
    assert len(audiofile.tag.frame_set[b"RVA2"]) == 4

    audiofile.tag.version = ID3_V2_3
    assert len(audiofile.tag.frame_set) == 1
    assert len(audiofile.tag.frame_set[b"RVAD"]) == 1
    assert audiofile.tag.frame_set[b"RVAD"][0].adjustments == \
        RelVolAdjFrameV23.VolumeAdjustments(front_left=20, front_right=19,back_left=-1,
                                            bass_peak=1024)


def test_RelVolAdjFrameV24_channel_type():
    for valid in range(9):
        RelVolAdjFrameV24().channel_type = valid
    for invalid in (-1, 9):
        with pytest.raises(ValueError):
            RelVolAdjFrameV24().channel_type = invalid


def test_RelVolAdjFrameV24_render_invalid_peak():
    rva2 = RelVolAdjFrameV24()
    rva2.peak = 2**32 - 1
    assert rva2.render()

    rva2.peak = 2**32
    with pytest.raises(ValueError):
        rva2.render()
    rva2.peak = 2**64
    with pytest.raises(ValueError):
        rva2.render()



import pytest
import unittest

from pathlib import Path
from unittest.mock import patch

import eyed3
from eyed3.id3 import (LATIN1_ENCODING, UTF_8_ENCODING, UTF_16_ENCODING,
                       UTF_16BE_ENCODING)
from eyed3.id3 import ID3_V1_0, ID3_V1_1, ID3_V2_3, ID3_V2_4
from eyed3.id3.frames import (Frame, TextFrame, FrameHeader, ImageFrame,
                              LanguageCodeMixin, ObjectFrame, TermsOfUseFrame,
                              DEFAULT_LANG, TOS_FID, OBJECT_FID)
from .. import DATA_D


class FrameTest(unittest.TestCase):
    def testCtor(self):
        f = Frame(b"ABCD")
        assert f.id == b"ABCD"
        assert f.header is None
        assert f.decompressed_size == 0
        assert f.group_id is None
        assert f.encrypt_method is None
        assert f.data is None
        assert f.data_len == 0
        assert f.encoding is None

        f = Frame(b"EFGH")
        assert f.id == b"EFGH"
        assert f.header is None
        assert f.decompressed_size == 0
        assert f.group_id is None
        assert f.encrypt_method is None
        assert f.data is None
        assert f.data_len == 0
        assert f.encoding is None

    def testTextDelim(self):
        for enc in [LATIN1_ENCODING, UTF_16BE_ENCODING, UTF_16_ENCODING,
                    UTF_8_ENCODING]:
            f = Frame(b"XXXX")
            f.encoding = enc
            if enc in [LATIN1_ENCODING, UTF_8_ENCODING]:
                assert (f.text_delim == b"\x00")
            else:
                assert (f.text_delim == b"\x00\x00")

    def testInitEncoding(self):
        # Default encodings per version
        for ver in [ID3_V1_0, ID3_V1_1, ID3_V2_3, ID3_V2_4]:
            f = Frame(b"XXXX")
            f.header = FrameHeader(f.id, ver)
            f._initEncoding()
            if ver[0] == 1:
                assert (f.encoding == LATIN1_ENCODING)
            elif ver[:2] == (2, 3):
                assert (f.encoding == UTF_16_ENCODING)
            else:
                assert (f.encoding == UTF_8_ENCODING)

        # Invalid encoding for a version is coerced
        for ver in [ID3_V1_0, ID3_V1_1]:
            for enc in [UTF_8_ENCODING, UTF_16_ENCODING, UTF_16BE_ENCODING]:
                f = Frame(b"XXXX")
                f.header = FrameHeader(f.id, ver)
                f.encoding = enc
                f._initEncoding()
                assert (f.encoding == LATIN1_ENCODING)

        for ver in [ID3_V2_3]:
            for enc in [UTF_8_ENCODING, UTF_16BE_ENCODING]:
                f = Frame(b"XXXX")
                f.header = FrameHeader(f.id, ver)
                f.encoding = enc
                f._initEncoding()
                assert (f.encoding == UTF_16_ENCODING)

        # No coersion for v2.4
        for ver in [ID3_V2_4]:
            for enc in [LATIN1_ENCODING, UTF_8_ENCODING, UTF_16BE_ENCODING,
                        UTF_16_ENCODING]:
                f = Frame(b"XXXX")
                f.header = FrameHeader(f.id, ver)
                f.encoding = enc
                f._initEncoding()
                assert (f.encoding == enc)


class TextFrameTest(unittest.TestCase):
    def testCtor(self):
        with pytest.raises(TypeError):
            TextFrame("TCON")

        f = TextFrame(b"TCON")
        assert f.text == ""

        f = TextFrame(b"TCON", "content")
        assert f.text == "content"

    def testRenderParse(self):
        fid = b"TPE1"
        for ver in [ID3_V2_3, ID3_V2_4]:
            h1 = FrameHeader(fid, ver)
            h2 = FrameHeader(fid, ver)
            f1 = TextFrame(b"TPE1", "Ambulance LTD")
            f1.header = h1
            data = f1.render()

            # FIXME: right here is why parse should be static
            f2 = TextFrame(b"TIT2")
            f2.parse(data[h1.size:], h2)
            assert f1.id == f2.id
            assert f1.text == f2.text
            assert f1.encoding == f2.encoding


class ImageFrameTest(unittest.TestCase):
    def testPicTypeConversions(self):
        count = 0
        for s in ("OTHER", "ICON", "OTHER_ICON", "FRONT_COVER", "BACK_COVER",
                  "LEAFLET", "MEDIA", "LEAD_ARTIST", "ARTIST", "CONDUCTOR",
                  "BAND", "COMPOSER", "LYRICIST", "RECORDING_LOCATION",
                  "DURING_RECORDING", "DURING_PERFORMANCE", "VIDEO",
                  "BRIGHT_COLORED_FISH", "ILLUSTRATION", "BAND_LOGO",
                  "PUBLISHER_LOGO"):
            c = getattr(ImageFrame, s)
            assert (ImageFrame.picTypeToString(c) == s)
            assert (ImageFrame.stringToPicType(s) == c)
            count += 1
        assert (count == ImageFrame.MAX_TYPE + 1)

        assert (ImageFrame.MIN_TYPE == ImageFrame.OTHER)
        assert (ImageFrame.MAX_TYPE == ImageFrame.PUBLISHER_LOGO)
        assert ImageFrame.picTypeToString(ImageFrame.MAX_TYPE) == \
               "PUBLISHER_LOGO"
        assert ImageFrame.picTypeToString(ImageFrame.MIN_TYPE) == "OTHER"

        with pytest.raises(ValueError):
            ImageFrame.picTypeToString(ImageFrame.MAX_TYPE + 1)
        with pytest.raises(ValueError):
            ImageFrame.picTypeToString(ImageFrame.MIN_TYPE - 1)

        with pytest.raises(ValueError):
            ImageFrame.stringToPicType("Prust")


def test_DateFrame():
    from eyed3.id3.frames import DateFrame
    from eyed3.core import Date

    # Default ctor
    df = DateFrame(b"TDRC")
    assert df.text == ""
    assert df.date is None

    # Ctor with eyed3.core.Date arg
    for d in [Date(2012),
              Date(2012, 1),
              Date(2012, 1, 4),
              Date(2012, 1, 4, 18),
              Date(2012, 1, 4, 18, 15),
              Date(2012, 1, 4, 18, 15, 30),
             ]:
        df = DateFrame(b"TDRC", d)
        assert df.text == str(d)
        # Comparison is on each member, not reference ID
        assert df.date == d

    # Test ctor str arg is converted
    for d in ["2012",
              "2010-01",
              "2010-01-04",
              "2010-01-04T18",
              "2010-01-04T06:20",
              "2010-01-04T06:20:15",
              "2012",
              "2010-01",
              "2010-01-04",
              "2010-01-04T18",
              "2010-01-04T06:20",
              "2010-01-04T06:20:15",
             ]:
        df = DateFrame(b"TDRC", d)
        dt = Date.parse(d)
        assert df.text == str(dt)
        assert df.text == str(d)
        # Comparison is on each member, not reference ID
        assert df.date == dt

    # Technically invalid, but supported
    for d in ["20180215"]:
        df = DateFrame(b"TDRC", d)
        dt = Date.parse(d)
        assert df.text == str(dt)
        # Comparison is on each member, not reference ID
        assert df.date == dt

    # Invalid dates
    for d in ["1234:12"]:
        date = DateFrame(b"TDRL")
        date.date = d
        assert not date.date


def test_compression():
    f = open(__file__, "rb")
    try:
        data = f.read()
        compressed = Frame.compress(data)
        assert data == Frame.decompress(compressed)
    finally:
        f.close()


'''
FIXME:
def test_tag_compression(id3tag):
    # FIXME: going to refactor FrameHeader, bbl
    data = Path(__file__).read_text()
    aframe = TextFrame(ARTIST_FID, text=data)
    aframe.header = FrameHeader(ARTIST_FID)
    import ipdb; ipdb.set_trace()
    pass
'''


def test_encryption():
    assert "Iceburn" == Frame.encrypt("Iceburn")
    assert "Iceburn" == Frame.decrypt("Iceburn")


def test_LanguageCodeMixin():
    with pytest.raises(TypeError):
        LanguageCodeMixin().lang = "eng"

    l = LanguageCodeMixin()
    l.lang = b"\x80"
    assert l.lang == b"eng"

    l.lang = b""
    assert l.lang == b""
    l.lang = None
    assert l.lang == b""


@pytest.mark.skipif(not Path(DATA_D).exists(), reason="test requires data files")
def test_TermsOfUseFrame(audiofile, id3tag):
    terms = TermsOfUseFrame()
    assert terms.id == b"USER"
    assert terms.text == ""
    assert terms.lang == DEFAULT_LANG

    id3tag.terms_of_use = "Fucking MANDATORY!"
    audiofile.tag = id3tag
    audiofile.tag.save()
    file = eyed3.load(audiofile.path)
    assert file.tag.terms_of_use == "Fucking MANDATORY!"

    id3tag.terms_of_use = "Fucking MANDATORY!"
    audiofile.tag = id3tag
    audiofile.tag.save()
    file = eyed3.load(audiofile.path)
    assert file.tag.terms_of_use == "Fucking MANDATORY!"

    id3tag.terms_of_use = ("Fucking MANDATORY!", b"jib")
    audiofile.tag = id3tag
    audiofile.tag.save()
    file = eyed3.load(audiofile.path)
    assert file.tag.terms_of_use == "Fucking MANDATORY!"
    assert file.tag.frame_set[TOS_FID][0].lang == b"jib"

    id3tag.terms_of_use = ("Fucking MANDATORY!", b"en")
    audiofile.tag = id3tag
    audiofile.tag.save()
    file = eyed3.load(audiofile.path)
    assert file.tag.terms_of_use == "Fucking MANDATORY!"
    assert file.tag.frame_set[TOS_FID][0].lang == b"en"


@pytest.mark.skipif(not Path(DATA_D).exists(), reason="test requires data files")
def test_ObjectFrame(audiofile, id3tag):
    sixsixsix = b"\x29\x0a" * 666
    with Path(__file__).open("rb") as fp:
        thisfile = fp.read()

    obj1 = ObjectFrame(description="Test Object", object_data=sixsixsix,
                       filename="666.txt", mime_type="text/satan")
    obj2 = ObjectFrame(description="Test Object2", filename=str(__file__),
                       mime_type="text/python", object_data=thisfile)
    id3tag.frame_set[OBJECT_FID] = obj1
    id3tag.frame_set[OBJECT_FID].append(obj2)

    audiofile.tag = id3tag
    audiofile.tag.save()
    file = eyed3.load(audiofile.path)
    assert len(file.tag.objects) == 2
    obj1_2 = file.tag.objects.get("Test Object")
    assert obj1_2.mime_type == "text/satan"
    assert obj1_2.object_data == sixsixsix
    assert obj1_2.filename == "666.txt"

    obj2_2 = file.tag.objects.get("Test Object2")
    assert obj2_2.mime_type == "text/python"
    assert obj2_2.object_data == thisfile
    assert obj2_2.filename == __file__


@pytest.mark.skipif(not Path(DATA_D).exists(), reason="test requires data files")
def test_ObjectFrame_no_mimetype(audiofile, id3tag):
    # Setting no mime-type is invalid
    obj1 = ObjectFrame(object_data=b"Deep Purple")
    id3tag.frame_set[OBJECT_FID] = obj1

    audiofile.tag = id3tag
    audiofile.tag.save()
    with patch("eyed3.core.parseError") as mock:
        file = eyed3.load(audiofile.path)
        assert mock.call_count == 2

    obj1.mime_type = "Deep"
    audiofile.tag.save()
    with patch("eyed3.core.parseError") as mock:
        file = eyed3.load(audiofile.path)
        assert mock.call_count == 1

    obj1.mime_type = "Deep/Purple"
    audiofile.tag.save()
    with patch("eyed3.core.parseError") as mock:
        file = eyed3.load(audiofile.path)
        mock.assert_not_called()

    obj1.object_data = b""
    audiofile.tag.save()
    with patch("eyed3.core.parseError") as mock:
        file = eyed3.load(audiofile.path)
        assert mock.call_count == 1
        assert file
